#!/usr/bin/python

import apt_pkg, os
from ubot.helper import UbotResponder, UbotCommander, main
os.environ['DJANGO_SETTINGS_MODULE'] = 'ubotweb.settings'
from ubotweb.encyclopedia.models import Factoid, SourcePackage, BinaryPackage
apt_pkg.init()

class PseudoFactoid(object):
    def __init__(self, txt):
        self.txt = txt
    def msg(self):
        return self.txt

class ErrorFactoid(PseudoFactoid):
    error = True

class EncyclopediaHelper(UbotResponder,UbotCommander):
    def in_privmsg(self, prefix, target, params):
        if not self.addressed(prefix, target, params[0]):
            return

        msg = params[0][1:].strip()

        # Target detection
        rprefix = ''
        rtarget = target
        if '|' in msg:
            msg, rprefix = [x.strip() for x in msg.split('|', 1)]
            rprefix += ': '
        elif '>' in msg and target.startswith('#'):
            msg, rtarget = [x.strip() for x in msg.split('>', 1)]
            rprefix = prefix[:prefix.find('!')] + ' wants you to know: '

        if rtarget != target and rtarget == prefix[:prefix.find('!')]:
            self.reply("Please use private messages to investigate. Searching can be done on " + self.conf.get(target, 'searchurl'), private=True)

        # Special cases first
        #if msg.startswith('ops '):
        #    return # Handled by other plugin

        elif msg.startswith('info '):
            factoid = self.package_info(msg[5:], target)
        elif msg.startswith('find ') or msg.startswith('search '):
            #factoid = self.search(msg[5:].strip(), target)
            factoid = ErrorFactoid("Please go to %s to search for packages and factoids" % self.conf.get(target, 'searchurl'))
        else:
          try:
            factoid = Factoid.objects.get(name=msg, channel=target)
          except Factoid.DoesNotExist:
            try:
              factoid = Factoid.objects.get(name=msg, channel='')
              factoid.popularity += 1
              factoid.save()
            except Factoid.DoesNotExist:
              factoid = self.package_info(msg, target)
              if hasattr(factoid, 'error'):
                  factoid = ErrorFactoid("Sorry, I do not know anything about %s. Please try searching on %s" % (msg, self.conf.get(target,'searchurl')))

        # Go.
        if hasattr(factoid, 'error') and factoid.error:
            self.reply(factoid.msg())
        else:
            if hasattr(factoid, 'alias_of'):
                i = 0
                while i < 5 and factoid.alias_of:
                    factoid = factoid.alias_of
                    factoid.popularity += 1
                    factoid.save()
                    i += 1
            self.send(rtarget, rprefix + factoid.msg())

    def package_info(self, msg, target):
        parts = msg.split()
        if len(parts) == 1:
            pkg, dis = parts[0], self.conf.get(target, 'release')
        else:
            pkg, dis = parts[:2]
        try:
            packages = list(BinaryPackage.objects.filter(packagename=pkg, release=dis))
            packages.sort(lambda x,y: apt_pkg.VersionCompare(x.version, y.version))
            return packages[0]
        except IndexError:
            # OK, do we have a sourcepackage?
            try:
                packages = list(SourcePackage.objects.filter(packagename=pkg, release=dis))
                packages.sort(lambda x,y: apt_pkg.VersionCompare(x.version, y.version))
                return packages[0]
            except IndexError:
                return ErrorFactoid("Package %s does not exist in %s" % (pkg, dis))

if __name__ == '__main__':
    main(EncyclopediaHelper)
