#!/usr/bin/python

import ConfigParser, optparse, os, signal, sys
import cPickle as pickle

parser = optparse.OptionParser(usage="%prog [options] <start|stop|restart|status> [helpername]")
parser.add_option('-c', '--config',
                  dest='config',
                  default=os.path.expanduser('~/.ubot/helpers'),
                  help="Specify configfile",
                  metavar="FILE")
parser.add_option('-d', '--dir',
                  dest='helperdir',
                  default='/usr/lib/ubot',
                  help="Where do the helpers live",
                  metavar="DIR")
parser.add_option('-f', '--no-fork',
                  dest='nofork',
                  action='store_true',
                  default=False,
                  help="Do not fork, stay in the foreground (works only with start and needs an argument)")

opts, args = parser.parse_args()
if len(args) not in (1,2) or args[0] not in ('start','stop','status','restart') :
    parser.error("Invalid number of arguments")
if opts.nofork and ((args[0] != 'start') or (len(args) != 2)):
    parser.error("Invalid number of arguments")
action = args[0]
helper = len(args) == 2 and args[1] or None

conffile = opts.config
statefile = opts.config + '.state'
state = {}
if os.path.exists(statefile):
    state = pickle.load(open(statefile))
state['pid'] = state.get('pid',{})
for h in state['pid'].keys():
    if not os.path.exists('/proc/%d' % state['pid'][h]):
        state['pid'].pop(h)

p = ConfigParser.ConfigParser()
p.read(conffile)

# Fetch all variables
vars={}
for o in p.options('variables'):
    vars[o] = p.get('variables',o,vars=vars)
# Find all helpers
helpers = {}
for s in p.sections():
    if s in ('all', 'variables'):
        continue
    helpers[s] = []
    vars['helpername'] = s
    for o in p.options('all'):
        helpers[s] += ['--' + o, p.get('all',o, vars=vars).replace('\n',',')]
    for o in p.options(s):
        helpers[s] += ['--' + o, p.get(s,o, vars=vars).replace('\n',',')]


if action == 'status':
    for h in helpers:
        if h not in state['pid']:
            print "Helper %s is not running" % h
        else:
            print "Helper %s is running, pid %d" % (h, state['pid'][h])
    sys.exit(0)

# What to do?
elif action in ('stop', 'restart'):
    if helper:
        if helper in state['pid']:
            print "Killing helper %s (%d)" % (helper, state['pid'][helper])
            os.kill(state['pid'].pop(helper), signal.SIGTERM)
        else:
            print >>sys.stderr, "E: Helper %s has not yet been started" % helper
    else:
        for h in state['pid']:
            print "Killing helper %s (%d)" % (h, state['pid'][h])
            os.kill(state['pid'][h], signal.SIGTERM)
        state['pid'] = {}
if action in ('start', 'restart'):
    if helper:
        if helper in state['pid']:
            print >>sys.stderr, "E: Helper %s has already been started" % helper
        else:
            if helper in helpers:
                hfile = helper.split(':')[0]
                argv = [os.path.join(opts.helperdir, hfile + '.py')] + helpers[helper]
                print "Starting %s helper %s" % (helper, str(argv))
                if opts.nofork:
                    os.execve(argv[0], argv, os.environ) 
                state['pid'][helper] = os.fork()
                if not state['pid'][helper]: # We're in the child
                    os.execve(argv[0], argv, os.environ) 
            else:
                print >>sys.stderr, "E: Unknown helper %s" % helper
    else:
        for h in helpers:
            if h not in state['pid']:
                hfile = h.split(':')[0]
                argv = [os.path.join(opts.helperdir, hfile + '.py')] + helpers[h]
                print "Starting %s helper %s" % (h, str(argv))
                state['pid'][h] = os.fork()
                if not state['pid'][h]: # We're in the child
                    os.execve(argv[0], argv, os.environ) 


# Write state file
pickle.dump(state, open(statefile, 'w'))
