import ubot.exceptions
import ubot.failover
import ubot.irc
import ConfigParser, os

class UbotConfig(object):
    def __init__(self, configfile):
        self.nicks       = []
        self.peers       = []
        self.servers     = []
        self.password    = None
        self.priority    = 1
        self.controlchan = None
        self.ident       = ubot.irc.IrcString('ubot')
        self.realname    = ubot.irc.IrcString('Ubot')
        self.busname     = os.path.basename(os.path.dirname(configfile))
        self.parse_config(configfile)
        self.datafile    = os.path.expanduser(os.path.join('~','.ubot',self.busname, 'data'))
        self.logfile     = os.path.expanduser(os.path.join('~','.ubot',self.busname, 'log'))
        self.ignorefile  = os.path.expanduser(os.path.join('~','.ubot',self.busname, 'ignores'))

    def parse_config(self, configfile):
        parser = ConfigParser.ConfigParser()
        parser.read(configfile)
        if not parser.has_section('me'):
            raise ubot.exceptions.ConfigError("No configuration found")
        i = 1
        while True:
            if parser.has_option('me', 'nick' + str(i)):
                self.nicks.append(ubot.irc.IrcString(parser.get('me', 'nick' + str(i))))
                i += 1
            else:
                break
        if not self.nicks:
            raise ubot.exceptions.ConfigError("No nicknames found")
        i = 1
        while True:
            if parser.has_option('me', 'server' + str(i)):
                self.servers.append(parser.get('me', 'server' + str(i)))
                i += 1
            else:
                break
        if not self.servers:
            raise ubot.exceptions.ConfigError("No servers found")
        for opt in ('ident', 'priority', 'password', 'realname', 'busname', 'controlchan'):
            if parser.has_option('me', opt):
                setattr(self, opt, parser.get('me', opt))
        self.ident = ubot.irc.IrcString(self.ident)
        self.realname = ubot.irc.IrcString(self.realname)
        self.controlchan = ubot.irc.IrcString(self.controlchan)
        self.priority = int(self.priority)

        # Parse peers
        i = 1
        while True:
            if parser.has_section('peer' + str(i)):
                self.peers.append(ubot.failover.UbotPeer(parser, 'peer' + str(i)))
                i += 1
            else:
                break
