import ubot.exceptions
import random, re

FAILOVER_HELLO = ("Ah, hello, Number Two",
                  "Ford! Hello, how are you?",
                  "Hi, Have you got a drink?",
                  "Bet you weren't expecting to see me again",
                  "Have you come far?",
                  "What...? Who...? When...? Where...?",
                  "Welcome to the starship heart of gold")

class UbotPeer(object):
    def __init__(self, config, section):
        self.is_active = False
        self.name = section
        if config.has_option(section, 'nickmatch'):
            self.nickmatch = re.compile(config.get(section, 'nickmatch'), re.I)
        else:
            raise ubot.exceptions.ConfigError("No nickmatch specified for %s" % section)
        if config.has_option(section, 'priority'):
            self.priority = int(config.get(section, 'priority'))
        else:
            raise ubot.exceptions.ConfigError("No priority specified for %s" % section)

    def isme(self, msg):
        return self.nickmatch.match(msg.prefix) != None

# Send messages when joining/seeing others join
def handle_join(msg, bot):
    for p in bot.config.peers:
        if p.isme(msg):
            bot.channels[msg.target].say(random.choice(FAILOVER_HELLO))

def handle_endofnames(msg, bot):
    if bot.synced:
        # We've returned after kick/part or joined for the first time
        bot.channels[msg.params[0]].say(random.choice(FAILOVER_HELLO))
        maybe_master(bot)

def handle_synced(msg, bot):
    bot.channels[bot.config.controlchan].say(random.choice(FAILOVER_HELLO))
    maybe_master(bot)

# Deactivate when leaving
def handle_part(msg, bot):
    if bot.isme(msg):
        return bot.set_master(False)
    for p in bot.config.peers:
        if p.isme(msg):
            p.is_active = False
            maybe_master(bot)

def handle_kick(msg, bot):
    if bot.isme(msg.params[0]):
        return bot.set_master(False)
    for p in bot.config.peers:
        if p.isme(msg.params[0]):
            p.is_active = False
            maybe_master(bot)

def handle_quit(msg, bot):
    for p in bot.config.peers:
        if p.isme(msg):
            p.is_active = False
            maybe_master(bot)

def handle_privmsg(msg, bot):
    for p in bot.config.peers:
        if p.isme(msg) and msg.params[0] in FAILOVER_HELLO:
            p.is_active = True
            maybe_master(bot)

def maybe_master(bot):
    # Not synced, not master
    if not bot.synced:
        return bot.set_master(False)
    # Am I the highest priority active bot
    minpri = bot.config.priority
    for p in bot.config.peers:
        if p.is_active:
            minpri = min(p.priority, minpri)
    if minpri == bot.config.priority:
        bot.set_master(True)
    else:
        bot.set_master(False)
