import ubot.irc
import ubot.log
import ubot.channel
import ubot.failover
import sys

# Join known channels
def handle_welcome(msg, bot):
    bot.nick = msg.target
    bot.loggedin = 2
    for c in bot.saved_channels:
        bot.join(c, bot.saved_passwords.get(c,''))

# Ping-pong with the server
def handle_ping(msg, bot):
    bot.ircconnection.fastsend(ubot.irc.OutMessage('PONG', msg.params))
def handle_pong(msg, bot):
    bot.ircconnection.ponged()

# Channel administration
def handle_join(msg, bot):
    if bot.isme(msg):
        ubot.log.log("Joined channel %s" % msg.target)
        bot.saved_channels.add(msg.target)
        bot.flush()
        bot.channels[msg.target] = ubot.channel.Channel(bot, msg.target)
        bot.ircconnection.slowsend(ubot.irc.OutMessage('MODE', [msg.target]))
    else:
        bot.channels[msg.target].nicks.append(msg.nick)
    ubot.failover.handle_join(msg, bot)

def handle_part(msg, bot):
    if bot.config.controlchan == msg.target:
        ubot.failover.handle_part(msg, bot)
    if not bot.isme(msg):
        bot.channels[msg.target].nicks.remove(msg.nick)
        return
    ubot.log.log("Left channel %s" % msg.target)
    c = bot.channels.pop(msg.target)
    c.remove_from_connection() # dbus connection, that is
    del(c)

def handle_kick(msg, bot):
    nick = msg.params[0]
    if bot.config.controlchan == msg.target:
        ubot.failover.handle_kick(msg, bot)
    if not bot.isme(nick):
        bot.channels[msg.target].nicks.remove(nick)
        return
    ubot.log.log("Was kicked from channel %s by %s" % (msg.target, msg.nick))
    c = bot.channels.pop(msg.target)
    c.remove_from_connection()
    del(c)
    bot.join(msg.target)

def _nomode(x):
    while x[0] in '@%+~':
        x = x[1:]
    return x
def handle_namreply(msg, bot):
    cname, nicks = msg.params[-2:]
    bot.channels[cname].nicks += [_nomode(x) for x in nicks.split()]

def handle_endofnames(msg, bot):
    cname = msg.params[0]
    bot.channels[cname].synced = True
    if cname == bot.config.controlchan:
        ubot.failover.handle_endofnames(msg, bot)

def handle_disconnect(msg, bot):
    bot.clear_data()

handle_synced = ubot.failover.handle_synced
def handle_privmsg(msg, bot):
    if msg.target == bot.config.controlchan:
        ubot.failover.handle_privmsg(msg, bot)

def handle_nick(msg, bot):
    if bot.isme(msg):
        bot.nick = msg.params[0]
    for c in bot.channels.values():
        if msg.nick in c.nicks:
            c.nicks.remove(msg.nick)
            c.nicks.append(msg.params[0])

def handle_quit(msg, bot):
    if msg.target == bot.config.controlchan:
        ubot.failover.handle_quit(msg, bot)
    for c in bot.channels.values():
        if msg.nick in c.nicks:
            c.nicks.remove(msg.nick)
            c.nicks.append(msg.params[0])

def handle_nicknameinuse(msg, bot):
    # Only time we should act on this is when bot.loggedin = 1 (tried login, not seen welcome)
    if bot.loggedin == 1:
        pos = bot.config.nicks.index(msg.params[0])
        if pos == len(bot.config.nicks) -1:
            ubot.log.log("Exhausted all nicknames")
            sys.exit(1)
        bot.ircconnection.send(ubot.irc.OutMessage('NICK', [bot.config.nicks[pos+1]]))

def handle_topic(msg, bot):
    bot.channels[msg.params[0]].topic = msg.params[1]

def handle_channelmodeis(msg, bot):
    bot.channels[msg.params[0]].mode = msg.params[1:]
