import dbus, os, sys
from django.conf import settings
from django.http import HttpResponse
from django.contrib.auth.decorators import user_passes_test
from django.db import models
from django.utils import simplejson
from django.shortcuts import render_to_response
from django.template import RequestContext
root = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
sys.path.insert(0,root)
import ubot.util

# Can't use a permission since there's no content_type to link it to
from django.contrib.auth.models import Group
masters, created = Group.objects.get_or_create(name="botmasters")

@user_passes_test(lambda u: u.is_superuser)
def overview(request):
    return render_to_response('control.html', context_instance=RequestContext(request))

class BotInfo(models.Model):
    synced = models.BooleanField()
    am_master = models.BooleanField()
    nick = models.CharField(maxlength=32)

@user_passes_test(lambda u: u.is_superuser)
def info(request):
    bot = dbus.SessionBus().get_object('net.seveas.ubot.' + settings.UBOT_BUSNAME, 
                                       '/net/seveas/ubot/' + settings.UBOT_BUSNAME)
    data = bot.get_info()
    return HttpResponse(simplejson.dumps(data), mimetype="text/x-json")

@user_passes_test(lambda u: u.is_superuser)
def channels(request):
    channels = []
    bot = dbus.SessionBus().get_object('net.seveas.ubot.' + settings.UBOT_BUSNAME, 
                                       '/net/seveas/ubot/' + settings.UBOT_BUSNAME)
    for c in bot.get_channels():
        _c = dbus.SessionBus().get_object('net.seveas.ubot.' + settings.UBOT_BUSNAME, 
                                          '/net/seveas/ubot/' + settings.UBOT_BUSNAME + '/channel/' + ubot.util.escape_object_path(c))
        channels.append({'name':c, 'mode':_c.get_mode(), 'topic':_c.get_topic(), 'nicks': _c.get_nicks()})
    return HttpResponse(simplejson.dumps(channels), mimetype="text/plain")

@user_passes_test(lambda u: u.is_superuser)
def join(request, name):
    bot = dbus.SessionBus().get_object('net.seveas.ubot.' + settings.UBOT_BUSNAME, 
                                       '/net/seveas/ubot/' + settings.UBOT_BUSNAME)
    bot.join('#' + name, '')
    return HttpResponse('Sent join command for #%s' % name, mimetype="text/plain")

@user_passes_test(lambda u: u.is_superuser)
def part(request, name):
    chan = dbus.SessionBus().get_object('net.seveas.ubot.' + settings.UBOT_BUSNAME, 
                                        '/net/seveas/ubot/' + settings.UBOT_BUSNAME + '/channel/' + ubot.util.escape_object_path('#' + name))
    chan.part("")
    return HttpResponse('Sent part command for #%s' % name, mimetype="text/plain")
