from django.core.cache import cache
from django.db.models import Q
from django.shortcuts import render_to_response
from django.template import RequestContext
from ubotweb.encyclopedia.models import Factoid, BinaryPackage, SourcePackage

def index(request):
    if not cache.get('nfactoids'):
        cache.set('nfactoids', Factoid.objects.count())
    if not cache.get('npackages'):
        cache.set('npackages', BinaryPackage.objects.count() + SourcePackage.objects.count())

    # Are we searching
    if 'q' in request.GET:
        words = request.GET['q'].split()[:3]
        if min([len(x) for x in words]) < 5:
            return render_to_response('encyclopedia/index.html',
                                      {'npackages': cache.get('npackages'),
                                       'nfactoids': cache.get('nfactoids'),
                                       'term': request.GET['q'],
                                       'error': "Please use longer searchterms"},
                                      context_instance=RequestContext(request))
        q1 = Q(packagename__contains=words[0]) | Q(shortdesc__contains=words[0])
        q2 = Q(packagename__contains=words[0])
        q3 = Q(name__contains=words[0]) | Q(value__contains=words[0])
        for term in words[1:]:
            q1 |= Q(packagename__contains=term) | Q(shortdesc__contains=term)
            q2 |= Q(packagename__contains=term)
            q3 |= Q(name__contains=term) | Q(value__contains=term)
        bpackages = BinaryPackage.objects.filter(q1).order_by('packagename','release')
        spackages = SourcePackage.objects.filter(q2).order_by('packagename','release')
        factoids = list(Factoid.objects.filter(q3).order_by('name','channel'))
        # Resolve aliases
        for i,f in enumerate(factoids):
            if f.alias_of:
                factoids[i] = f.alias_of
        factoids = list(set(factoids))
        factoids.sort()
        factoids.sort(key=lambda x: (x.name, x.channel))

        return render_to_response('encyclopedia/search.html',
                                  {'term': request.GET['q'],
                                   'bpackages': bpackages,
                                   'spackages': spackages,
                                   'factoids': factoids},
                                  context_instance=RequestContext(request))

    # Most popular
    if not cache.get('mostpopular'):
        cache.set('mostpopular', Factoid.objects.order_by('-popularity')[:20])
    return render_to_response('encyclopedia/index.html',
                              {'npackages': cache.get('npackages'),
                               'nfactoids': cache.get('nfactoids'),
                               'mostpopular': cache.get('mostpopular')},
                              context_instance=RequestContext(request))

def factoid(request, name):
    bpackages = BinaryPackage.objects.filter(packagename=name).order_by('release')
    spackages = SourcePackage.objects.filter(packagename=name).order_by('release')
    factoids = Factoid.objects.filter(name=name).order_by("channel")

    return render_to_response('encyclopedia/factoid.html',
                              {'name': name,
                               'bpackages': bpackages,
                               'spackages': spackages,
                               'factoids': factoids},
                              context_instance=RequestContext(request))
